// LevelHandler.cpp: implementation of the LevelHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Sprite.h"
#include "Camera.h"
#include "Texture.h"
#include "Monster.h"
#include "Zombie.h"
#include "Ninja.h"
#include "Pirate.h"
#include "Robot.h"
#include "Knife.h"
#include "Cabbage.h"
#include "LevelHandler.h"
#include "MenuHandler.h"
#include "WinHandler.h"
#include "TransitionHandler.h"
#include "HealthBox.h"

#define MAX_LEVEL 15

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LevelHandler::LevelHandler()
{
	
}

LevelHandler::~LevelHandler()
{
	//Cleanup
	knifeTex.Unload();
	background.texture.Unload();
	healthBox.Unload();
	for(int i = 0; i < pepper.size(); i++)
	{
		delete pepper[i];
	}
	pepper.clear();
	for(i = 0; i < monsters.size(); i++)
	{
		delete monsters[i];
	}
	missiles.clear();
	monsters.clear();
	pirateTex[0].Unload();
	zombieTex.Unload();
	knifeTex.Unload();
	flameThrower.Unload();
	fire.Unload();
	for(i = 0; i < 4; i++)
		ninjaTex[i].Unload();
	for(i = 0; i < 5; i++)
		zombieAttack[i].Unload();
}

void LevelHandler::Update()
{
	if(!bWin && !bDead && !bText)
	{
		//Check for player->pepper collision
		for(int i = 0; i < pepper.size(); i++)
		{
			if(!pepper[i]->visible)
				continue;
			if(Collide(*pepper[i], player))
			{
				nPoints += nPointsPerPepper;
				pepper[i]->visible = false;
				nPeppers--;
			}
		}

		//Update enemies
		for(i = 0; i < monsters.size(); i++)
		{
			hp -= monsters[i]->Move(player.xPos, player.yPos);
		}

		//Update missiles
		list<Monster*>::iterator it;
		for(it = missiles.begin(); it != missiles.end(); it++)
		{
			hp -= (*it)->Move(player.xPos, player.yPos);
			if((*it)->sprite.visible == false && missiles.size() > 1)
				it = missiles.erase(it);
		}

		//Update health boxes
		for(i = 0; i < hBoxes.size(); i++)
		{
			hp += hBoxes[i].Update(player);
		}
		if(hp > START_HP)
			hp = START_HP;

		//Check for player->screen collisions
		float left;
		float right;
		float top;
		float bottom;

		left = player.xPos - player.width * 0.5f;
		right = player.xPos + player.width * 0.5f;
		top = player.yPos + player.height * 0.5f;
		bottom = player.yPos - player.height * 0.5f;

		if(left < -100.0f)
			player.xPos = -100.0f + player.width * 0.5f;
		if(right > 100.0f)
			player.xPos = 100.0f - player.width * 0.5f;
		if(bottom < -100.0f)
			player.yPos = -100.0f + player.height * 0.5f;
		if(top > 100.0f)
			player.yPos = 100.0f - player.height * 0.5f;
	}

	//Check for end-of-level and redden Mr.Green
	if(nPeppers == 0 && !bDead)
	{
		bWin = true;
		bLevelLoaded = false;
		winTimer = -50;
		player.colour.b = 255;
		player.colour.g = 255;
		nPeppers = pepper.size();
		nLevel++;
		if(nLevel > MAX_LEVEL)
		{
			TransitionHandler* t = new TransitionHandler;
			WinHandler* winH = new WinHandler;
			winH->points = nPoints;
			winH->Initialize();
			t->BeginTransition(this, winH, 25);
			state = t;
			bEnd = true;
			return;
		}
	}else{
		float redness = (float)nPeppers / (float)pepper.size();
		player.colour.b = redness * 255;
		player.colour.g = redness * 255;
	}
}

void LevelHandler::Render()
{
	//Setup the camera
	cam.Render();

	if(!bText && !bEnd)
	{
		glDisable(GL_LIGHTING);
		player.Render();
	}

	glEnable(GL_LIGHTING);

	if(!bWin && !bDead && !bText)
	{
		//Render peppers
		for(int i = 0; i < pepper.size(); i++)
		{
			pepper[i]->Render();
		}
	
		//Render enemies
		for(i = 0; i < monsters.size(); i++)
		{
			monsters[i]->Render();
		}

		//Render missiles
		list<Monster*>::iterator it;
		for(it = missiles.begin(); it != missiles.end(); it++)
		{
			(*it)->Render();
		}

		//Render Health Boxes
		for(i = 0; i < hBoxes.size(); i++)
		{
			hBoxes[i].sprite.Render();
		}

		//Render health bar
		float per = hp / (float)START_HP;
		float w = per * 15;
		hpbar.texture.uMax = per;
		hpbar.width = w;
		hpbar.xPos = -100.0f + hpbar.width * 0.5f + 2.0f;
		hpbar.Render();
	}

	if(bWin && !bEnd)
	{
		winTimer += 80.0f * m;
		float ambient = fabs(winTimer) / 50.0f;
		if(winTimer >= 50.0f)
		{
			ambient = 1.0f;
			bWin = false;
		}
		if(winTimer >= 0 && !bLevelLoaded)
		{
			LoadLevel(nLevel);
		}
		float light[] = {ambient, ambient, ambient, 1.0f};
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light);
	}

	//Check for death
	if(hp <= 0 && !bEnd)
	{
		hp = 0;
		if(bDead)
		{
			winTimer += 40.0f * m;
			float ambient = fabs(winTimer) / 50.0f;
			if(winTimer >= 50.0f)
			{
				ambient = 1.0f;
				bDead = false;
				hp = START_HP;
			}
			if(ambient < 0.75f)
			{
				dead.width = 25;
				dead.height = 12.5f;
			}
			if(ambient < 0.6f)
			{
				dead.width = 50;
				dead.height = 25;
			}
			if(ambient < 0.35f)
			{
				dead.width = 75;
				dead.height = 37.5f;
			}
			if(winTimer >= 0)
			{
				dead.visible = false;
				MenuHandler* menu = new MenuHandler;
				menu->Initialize();
				menu->LoadLevel(1);
				state = menu;
				return;
			}
			float light[] = {ambient, ambient, ambient, 1.0f};
			glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light);
			dead.Render();
		}else{
			nLevel = 1;
			dead.visible = true;
			bDead = true;
			bLevelLoaded = false;
			winTimer = -50;
			player.colour.b = 255;
			player.colour.g = 255;
		}
	}

	//Render the background
	background.Render();

	//Render text
	if(bText)
	{
		glDisable(GL_LIGHTING);
		storyText.Render(-75, 75);
	}
}

void LevelHandler::LoadLevel(int level)
{
	//Unload the previous level
	background.texture.Unload();
	for(int i = 0; i < pepper.size(); i++)
	{
		delete pepper[i];
	}
	pepper.clear();
	for(i = 0; i < monsters.size(); i++)
	{
		delete monsters[i];
	}
	missiles.clear();
	monsters.clear();
	ninjas.clear();
	hBoxes.clear();

	//Load the current level
	bLevelLoaded = true;
	char buf[255] = {0};
	string s = "Levels\\l";
	itoa(level, buf, 10);
	s += buf;
	s += ".lvl";
	ifstream file;
	file.open(s.c_str());

	file.getline(buf, 255);
	int nPirates = atoi(buf);

	file.getline(buf, 255);
	int nNinjas = atoi(buf);

	file.getline(buf, 255);
	int nRobots = atoi(buf);

	file.getline(buf, 255);
	int nZombies = atoi(buf);

	file.getline(buf, 255);
	int nCabbages = atoi(buf);

	file.getline(buf, 255);
	int nHealth = atoi(buf);

	file.getline(buf, 255);
	nPeppers = atoi(buf);

	file.getline(buf, 255);
	nPointsPerPepper = atoi(buf);

	file.getline(buf, 255);	//Background tex

	//Get the background texture name
	s = "Textures\\";
	s += buf;

	//Load the background sprite
	background.texture.Load(s.c_str());
	background.width = 200;
	background.height = 200;
	background.zDepth = 9;

	//Load the story text
	file.getline(buf, 256);
	s = buf;
	if(!(s == "none"))
	{
		//Get text colour
		file.getline(buf, 255);
		if(!string(buf).empty())
		{
			int r = 0;
			int g = 0;
			int b = 0;
			sscanf(buf, "%d %d %d", &r, &g, &b);
			if(r > 255)
				r = 255;
			if(r < 0)
				r = 0;
			if(g > 255)
				g = 255;
			if(g < 0)
				g = 0;
			if(b > 255)
				b = 255;
			if(b < 0)
				b = 0;
			storyText.colour.Set((char)r, (char)g, (char)b, (char)255);
		}else{
			storyText.colour.Set((char)255, (char)255, (char)255, (char)255);
		}

		//Load the text
		bText = true;
		storyText.Clear();
		storyText.zDepth = 1;
		storyText.LoadFromFile((char*)s.c_str());
	}

	file.close();

	//Load the player sprite
	player.texture = playerTex;
	player.width = 10;
	player.height = 10;
	player.zDepth = 3;

	//Load pepper sprites
	srand(GetTickCount());
	for(i = 0; i < nPeppers; i++)
	{
		pepper.push_back(new Sprite);
		pepper[i]->texture = pepTex;
		pepper[i]->width = 10;
		pepper[i]->height = 10;
		pepper[i]->zDepth = 5;
		pepper[i]->xPos = (rand() % 180) - 90;
		pepper[i]->yPos = (rand() % 180) - 90;
	}

	//Load health boxes
	for(i = 0; i < nHealth; i++)
	{
		HealthBox hb;
		hb.sprite.zDepth = 4;
		hb.sprite.texture = healthBox;
		hb.Initialize();
		hBoxes.push_back(hb);
	}

	//Load zombies
	for(i = 0; i < nZombies; i++)
	{
		Zombie* z = new Zombie;
		z->zombieTex = zombieTex;
		z->Setup();
		z->sprite.width = 32;
		z->sprite.height = 32;
		z->sprite.zDepth = 2;
		z->sprite.xPos = (rand() % 180) - 90;
		z->sprite.yPos = (rand() % 180) - 90;
		z->fire.texture = fire;

		z->zombieAttack.push_back(zombieAttack[0]);
		z->zombieAttack.push_back(zombieAttack[1]);
		z->zombieAttack.push_back(zombieAttack[2]);
		z->zombieAttack.push_back(zombieAttack[3]);
		z->zombieAttack.push_back(zombieAttack[4]);

		monsters.push_back(z);
	}

	//Load ninjas
	for(i = 0; i < nNinjas; i++)
	{
		Ninja* n = new Ninja;
		n->sprite.texture = ninjaTex[0];
		n->sprite.width = 32;
		n->sprite.height = 32;
		n->sprite.zDepth = 2;
		n->sprite.xPos = (rand() % 180) - 90;
		n->sprite.yPos = (rand() % 180) - 90;
		n->Setup();

		n->ninjaTex.push_back(ninjaTex[0]);
		n->ninjaTex.push_back(ninjaTex[1]);
		n->ninjaTex.push_back(ninjaTex[2]);
		n->ninjaTex.push_back(ninjaTex[3]);
		n->ninjaTex.push_back(ninjaTex[4]);

		n->priority = i;

		n->peers = &ninjas;

		ninjas.push_back(n);
		monsters.push_back(n);
	}

	//Load cabbages
	for(i = 0; i < nCabbages; i++)
	{
		Cabbage* c = new Cabbage;
		c->tex[0] = cabbageTex[0];
		c->tex[1] = cabbageTex[1];
		c->tex[2] = cabbageTex[2];
		c->Setup();
		monsters.push_back(c);
	}

	//Load pirates
	for(i = 0; i < nPirates; i++)
	{
		Pirate* p = new Pirate;
		p->sprite.width = 32;
		p->sprite.height = 32;
		p->sprite.zDepth = 2;
		p->sprite.xPos = (rand() % 180) - 90;
		p->sprite.yPos = (rand() % 180) - 90;
		p->sprite.texture = pirateTex[0];
		p->monsters = &missiles;
		p->knifeTex = knifeTex;
		p->Setup();
		
		monsters.push_back(p);
	}

	//Load robots
	for(i = 0; i < nRobots; i++)
	{
		Robot* r = new Robot;
		r->flame.texture = flameThrower;
		r->monsters = &monsters;
		r->sprite.texture = robotTex;
		r->sprite.width = 32;
		r->sprite.height = 32;
		r->sprite.zDepth = 2;
		r->sprite.xPos = (rand() % 180) - 90;
		r->sprite.yPos = (rand() % 180) - 90;
		r->Setup();

		monsters.push_back(r);
	}
}

void LevelHandler::OnKeys(bool keys[])
{
	//Move the player
	if(!bWin && !bDead)
	{
		if(keys[VK_UP])
			player.yPos += m * 80.0f;
		if(keys[VK_DOWN])
			player.yPos -= m * 80.0f;
		if(keys[VK_RIGHT])
			player.xPos += m * 80.0f;
		if(keys[VK_LEFT])
			player.xPos -= m * 80.0f;
		if(keys[VK_RETURN])
			bText = false;

		//Transition to the main menu on ESC
		if(keys[VK_ESCAPE] || keys[VK_HOME])
		{
			MenuHandler* menu = new MenuHandler;
			menu->retLevel = new LevelHandler;
			menu->retLevel->Initialize();
			menu->retLevel->nLevel = nLevel;
			menu->retLevel->LoadLevel(nLevel);
			menu->retLevel->bText = bText;
			menu->retLevel->hp = hp;
			menu->retLevel->nPoints = nPoints;
			menu->retLevel->nPeppers = nPeppers;
			menu->retLevel->player.xPos = player.xPos;
			menu->retLevel->player.yPos = player.yPos;
			for(int i = 0; i < monsters.size(); i++)
			{
				int type = monsters[i]->GetType();
				if(type == 1)
				{
					//Zombie
					Zombie* old = (Zombie*)monsters[i];
					Zombie* z = (Zombie*)menu->retLevel->monsters[i];
					z->destX = old->destX;
					z->destY = old->destY;
					z->sprite.xPos = old->sprite.xPos;
					z->sprite.yPos = old->sprite.yPos;
				}else if(type == 2)
				{
					//Ninja
					Ninja* old = (Ninja*)monsters[i];
					Ninja* n = (Ninja*)menu->retLevel->monsters[i];
					n->bDir = old->bDir;
					n->destX = old->destX;
					n->destY = old->destY;
					n->priority = old->priority;
					n->sprite.xPos = old->sprite.xPos;
					n->sprite.yPos = old->sprite.yPos;
				}else if(type == 3)
				{
					//Pirate
					Pirate* old = (Pirate*)monsters[i];
					Pirate* p = (Pirate*)menu->retLevel->monsters[i];
					p->attackTimer = old->attackTimer;
					p->destX = old->destX;
					p->destY = old->destY;
					p->sprite.xPos = old->sprite.xPos;
					p->sprite.yPos = old->sprite.yPos;
				}else if(type == 4)
				{
					//Robot
					Robot* old = (Robot*)monsters[i];
					Robot* r = (Robot*)menu->retLevel->monsters[i];
					r->angle = old->angle;
					r->attackTimer = old->attackTimer;
					r->destX = old->destX;
					r->destY = old->destY;
					r->sprite.xPos = old->sprite.xPos;
					r->sprite.yPos = old->sprite.yPos;
				}else if(type == 5)
				{
					//Cabbage
					Cabbage* old = (Cabbage*)monsters[i];
					Cabbage* c = (Cabbage*)menu->retLevel->monsters[i];
					c->attackTimer = old->attackTimer;
					c->sprite.xPos = old->sprite.xPos;
					c->sprite.yPos = old->sprite.yPos;
					c->timer = old->timer;
				}
			}
			list<Monster*>::iterator it;
			for(it = missiles.begin(); it != missiles.end(); it++)
			{
				Monster* mon = (*it);
				if(mon->GetType() == 128)
				{
					Knife* k = new Knife;
					Knife* old = (Knife*)mon;
					k->texture = menu->retLevel->knifeTex;
					k->xSpeed = old->xSpeed;
					k->ySpeed = old->ySpeed;
					k->sprite.xPos = old->sprite.xPos;
					k->sprite.yPos = old->sprite.yPos;
					k->monsters = &menu->retLevel->missiles;
					k->Setup();
					menu->retLevel->missiles.push_back(k);
				}
			}
			for(i = 0; i < pepper.size(); i++)
			{
				menu->retLevel->pepper[i]->visible = pepper[i]->visible;
				menu->retLevel->pepper[i]->xPos = pepper[i]->xPos;
				menu->retLevel->pepper[i]->yPos = pepper[i]->yPos;
			}
			for(i = 0; i < hBoxes.size(); i++)
			{
				menu->retLevel->hBoxes[i] = hBoxes[i];
			}
			menu->Initialize();
			menu->LoadLevel(2);

			TransitionHandler* t = new TransitionHandler;
			t->Initialize();
			t->BeginTransition(this, menu, 15);
			state = t;
		}
	}
}

void LevelHandler::Initialize()
{
	bEnd = false;
	dead.texture.Load("Textures\\dead.png");
	dead.zDepth = 0.5f;
	dead.width = 25;
	dead.height = 12.5f;

	hp = START_HP;
	hpbar.zDepth = 1;
	hpbar.texture.Load("Textures\\HealthBar.png");
	hpbar.height = 3;
	hpbar.width = 25;
	hpbar.xPos = -100 + 12.5f;
	hpbar.yPos = 97;

	nLevel = 1;
	bWin = false;
	bDead = false;
	nPoints = 0;
	float light[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light);
	glMaterialfv(GL_FRONT, GL_AMBIENT, light);

	flameThrower.Load("Textures/flame.png");
	fire.Load("Textures/fire.png");
	pepTex.Load("Textures/pepper.png");
	playerTex.Load("Textures/player.gif");
	zombieTex.Load("Textures/Zombie1.png", 
		Colour((char)255, (char)0, (char)255, (char)255));
	knifeTex.Load("Textures/knife.png");
	robotTex.Load("Textures/robot.png");

	cabbageTex[0].Load("Textures/cabbage1.png");
	cabbageTex[1].Load("Textures/cabbage2.png");
	cabbageTex[2].Load("Textures/cabbage.png");

	nPeppers = 0;

	healthBox.Load("Textures/Health.png");

	string s;
	char buf[255] = {0};
	for(int i = 0; i < 5; i++)
	{
		s = "Textures\\Zombie";
		s += itoa(i + 1, buf, 255);
		s += ".png";
		zombieAttack[i].Load(s.c_str(), Colour((char)255, (char)0, (char)255, (char)255));
	}

	for(i = 0; i < 4; i++)
	{
		s = "Textures\\Ninja";
		s += itoa(i + 1, buf, 255);
		s += ".png";
		ninjaTex[i].Load(s.c_str(), Colour((char)255, (char)0, (char)255, (char)255));
	}
	for(i = 0; i < 1; i++)
	{
		s = "Textures\\Pirate";
		s += itoa(i + 1, buf, 255);
		s += ".png";
		pirateTex[i].Load(s.c_str());
	}
}

void LevelHandler::SaveTo(string s)
{
	//Open the file for writing
	ofstream file;
	file.open(s.c_str());

	//Write header info
	file.write((char*)&nLevel, sizeof(int));
	file.write((char*)&hp, sizeof(int));
	file.write((char*)&nPoints, sizeof(int));

	//Write whether or not the level has actually been loaded
	file.write((char*)&bText, sizeof(bool));

	//Close the file
	file.close();
}
