// LoadHandler.cpp: implementation of the LoadHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Camera.h"
#include "Sprite.h"
#include "TransitionHandler.h"
#include "LevelHandler.h"
#include "MenuHandler.h"
#include "LoadHandler.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LoadHandler::LoadHandler()
{

}

LoadHandler::~LoadHandler()
{
	upScroll.texture.Unload();
	scrollButton.texture.Unload();
	loadButton.texture.Unload();
	bar.texture.Unload();
}

void LoadHandler::Initialize()
{
	//Get info on each saved file
	_finddata_t fd;
	long hFile;
	hFile = _findfirst("Save\\*.sav", &fd);
	if(hFile == -1L)
	{
		bFiles = false;
	}else{
		int iRes = 0;
		while(iRes == 0)
		{
			AddFile(fd);
			iRes = _findnext(hFile, &fd);
		}
		_findclose(hFile);
	}

	cam.viewWidth = 200;
	cam.viewHeight = 200;
	curFile = 0;
	if(slots.size() == 0)
		bFiles = false;

	text.colour.Set((char)0, (char)255, (char)255, (char)255);

	//Create the scroll button and arrows
	upScroll.texture.Load("Textures/arrow.png", Colour(0,0,0,0));
	upScroll.xPos = 90;
	upScroll.width = 5;
	upScroll.height = 5;
	upScroll.yPos = 75;

	downScroll.texture = upScroll.texture;
	downScroll.rotation = 180;
	downScroll.xPos = 90;
	downScroll.width = 5;
	downScroll.height = 5;
	downScroll.yPos = -75;

	scrollButton.texture.Load("Textures/bar.png", Colour(0,0,0,0));
	scrollButton.xPos = 90;
	scrollButton.width = 5;
	scrollButton.height = 5;
	scrollButton.yPos = 70;

	//Create the Load button and return button
	loadButton.texture.Load("Textures/loadgame.png");
	loadButton.xPos = 50;
	loadButton.yPos = -50;
	loadButton.width = 60;
	loadButton.height = 20;

	returnButton.texture.Load("Textures/returngame.png");
	returnButton.xPos = -50;
	returnButton.yPos = -50;
	returnButton.width = 60;
	returnButton.height = 20;

	bar.texture.Load("Textures/white.bmp");
	bar.width = 30;
	bar.visible = false;
	bar.height = 2;

	//Set the control focus to nothing
	focus = 0;

}

void LoadHandler::Render()
{
	glDisable(GL_LIGHTING);
	cam.Render();
	if(!bFiles)
	{
		text.text = "Couldn't find any saved files.";
		text.text += '\n';
		text.Render(-25,0);
	}else{

		//Render game slots
		int nMax = curFile + 5;
		int nMin = curFile - 5;
		if(nMin < 0)
		{
			nMax = 10;
			nMin = 0;
		}else if(nMax > slots.size())
		{
			nMin = slots.size() - 10;
			nMax = slots.size();
		}
		if(nMin < 0)
			nMin = 0;
		if(nMax > slots.size())
			nMax = slots.size();
		text.text = "";
		
		for(int i = nMin; i < nMax; i++)
		{
			if(curFile == i)
				text.text += "-> ";
			text.text += slots[i].text; 
			text.text += '\n';
		}
		text.Render(-75, 75);

		//Render the scroller arrows
		upScroll.Render();
		downScroll.Render();

		//Calculate scroll button position & dimensions
		float hPer = 1.0f / (float)(slots.size());
		scrollButton.height = hPer * 150.0f;
		scrollButton.yPos = (hPer * (float)(slots.size() - 1 - curFile) * 
			(140.0f)) - (70.0f - scrollButton.height * 0.5f);
		scrollButton.Render();

		//Render other buttons
		loadButton.Render();
	}
	returnButton.Render();
	bar.Render();
}

void LoadHandler::AddFile(_finddata_t &fd)
{
	//Open the file and get it's header info
	ifstream file;
	string s = "Save/";
	s += fd.name;
	file.open(s.c_str());

	FileInfo fi;

	//Read level #
	int nLevel = 0;
	file.read((char*)&nLevel, sizeof(int));

	//Read HP
	int hp = 0;
	file.read((char*)&hp, sizeof(int));

	//Read score
	int score = 0;
	file.read((char*)&score, sizeof(int));

	//Read bText
	file.read((char*)&fi.bText, sizeof(bool));

	fi.score = score;
	fi.hp = hp;
	fi.nLevel = nLevel;

	//Construct text
	char buf[256] = {0};
	sprintf(buf, "%s   Level: %d   %d/30 HP   Score: %d",
		fd.name, nLevel, hp, score);
	fi.name = fd.name;
	fi.text = buf;
	slots.push_back(fi);

	//Close the file
	file.close();
}

void LoadHandler::OnKeyDown(int key)
{
	if(key == VK_DOWN)
		curFile++;
	if(key == VK_UP)
		curFile--;

	if(curFile >= (int)slots.size())
		curFile = 0;
	if(curFile < 0)
		curFile = (int)slots.size() - 1;
}

void LoadHandler::OnMouse(float x, float y)
{
	x = x * 200.0f - 100.0f;
	y = y * 200.0f - 100.0f;

	if(Collide(loadButton, x, y) && loadButton.visible)
	{
		focus = 1;
		bar.visible = true;
		bar.xPos = loadButton.xPos;
		bar.yPos = loadButton.yPos - 9;
	}else if(Collide(returnButton, x, y))
	{
		focus = 2;
		bar.visible = true;
		bar.xPos = returnButton.xPos - 10;
		bar.yPos = returnButton.yPos - 9;
	}else{
		focus = 0;
		bar.visible = false;
	}
}

void LoadHandler::OnMouseUp(bool lmb, bool rmb)
{
	if(!lmb)
		return;
	if(focus == 1)
	{
		//Create a level from file and transition to it
		FileInfo& fi = slots[curFile];
		LevelHandler* l = new LevelHandler;
		l->Initialize();
		l->LoadLevel(fi.nLevel);
		l->hp = fi.hp;
		l->bText = fi.bText;
		l->nPoints = fi.score;
		l->nLevel = fi.nLevel;
		TransitionHandler* t = new TransitionHandler;
		t->BeginTransition(this, l, 10);
		state = t;
	}else if(focus == 2)
	{
		//Transition to the main menu
		MenuHandler* menu = new MenuHandler;
		TransitionHandler* t = new TransitionHandler;
		
		menu->Initialize();
		if(bInGame)
			menu->LoadLevel(2);
		else
			menu->LoadLevel(1);
		menu->retLevel = lvl;
		t->BeginTransition(this, menu, 15);
		state = t;
	}
}
