// MenuHandler.cpp: implementation of the MenuHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Sprite.h"
#include "Camera.h"
#include "Texture.h"
#include "TextRenderer.h"
#include "MenuHandler.h"
#include "TransitionHandler.h"
#include "StoryHandler.h"
#include "LevelHandler.h"
#include "LoadHandler.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Texture MenuHandler::bkTex;

MenuHandler::MenuHandler()
{
	bSaveGame = false;
	if(bkTex.textureID <= 0)
		bkTex.Load("Textures/title.png", Colour(0,0,0,0));
}

MenuHandler::~MenuHandler()
{
	bar.texture.Unload();
	loadGame.texture.Unload();
	newGame.texture.Unload();
	story.texture.Unload();
	quitGame.texture.Unload();
	if(returnGame.visible)
		returnGame.texture.Unload();
	if(saveGame.visible)
		saveGame.texture.Unload();
}

void MenuHandler::Render()
{
	glEnable(GL_LIGHTING);
	float light[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT, light);

	cam.Render();

	background.Render();
	if(!bSaveGame)
	{
		bar.Render();
		newGame.Render();
		loadGame.Render();
		story.Render();
		quitGame.Render();
		returnGame.Render();
		saveGame.Render();
	}else{
		//Create a string to display then render it
		glDisable(GL_LIGHTING);
		string s = saveName;
		s.insert(caretPos, "|");
		saveNameRenderer.colour.Set((char)0, (char)255, (char)255, (char)255);
		saveNameRenderer.text = "Save To: ";
		saveNameRenderer.text += s;
		saveNameRenderer.text += '\n';
		saveNameRenderer.Render(-50,-10);
	}
}

void MenuHandler::Initialize()
{
	cam.viewWidth = 200;
	cam.viewHeight = 200;
	background.texture = bkTex;
	background.width = 200;
	background.height = 200;
	background.xPos = 0;
	background.yPos = 0;
	background.zDepth = 10;
}

void MenuHandler::LoadLevel(int level)
{
	bSaveGame = false;

	float light[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light);
	glMaterialfv(GL_FRONT, GL_AMBIENT, light);

	bar.texture.Load("Textures/dot.gif", Colour(0,0,0,0));
	bar.width = 35;
	bar.height = 2;
	bar.xPos = 0;
	bar.visible = false;

	newGame.texture.Load("Textures/NewGame.png");
	newGame.xPos = 5;
	newGame.yPos = -15;
	newGame.width = 50;
	newGame.height = 25;

	loadGame.texture.Load("Textures/LoadGame.png");
	loadGame.xPos = 5;
	loadGame.yPos = -35;
	loadGame.width = 50;
	loadGame.height = 25;

	story.texture.Load("Textures/GameStory.png");
	story.xPos = 12;
	story.yPos = -55;
	story.width = 50;
	story.height = 25;

	quitGame.texture.Load("Textures/quitgame.png");
	quitGame.xPos = 5;
	quitGame.yPos = - 75;
	quitGame.width = 50;
	quitGame.height = 25;

	returnGame.visible = false;
	saveGame.visible = false;
	bInGame = false;
	if(level == 2)
	{
		bInGame = true;

		//Create a Return To Game button on the left and Save Game on the right
		returnGame.texture.Load("Textures/ReturnGame.png");
		returnGame.xPos = -50;
		returnGame.yPos = -25;
		returnGame.width = 50;
		returnGame.height = 25;
		returnGame.visible = true;

		saveGame.texture.Load("Textures/SaveGame.png");
		saveGame.xPos = 55;
		saveGame.yPos = -25;
		saveGame.width = 50;
		saveGame.height = 25;
		saveGame.visible = true;
	}

}

void MenuHandler::OnMouse(float x, float y)
{
	//Convert from screen % to world space coordinates
	x = (x * 200.0f) - 100.0f;
	y = (y * 200.0f) - 100.0f;

	//Check if the mouse is over any of the buttons
	if(Collide(newGame, x, y))
	{
		bar.visible = true;
		bar.yPos = -27.5f;
		bar.xPos = 0;
		focus = 1;
	}else if(Collide(loadGame, x, y))
	{
		bar.visible = true;
		bar.yPos = -47.5f;
		bar.xPos = 0;
		focus = 2;
	}else if(Collide(story, x, y)){
		bar.visible = true;	
		bar.yPos = -67.5f;
		bar.xPos = 0;
		focus = 3;
	}else if(Collide(quitGame, x, y)){
		bar.visible = true;
		bar.yPos = -87.5f;
		bar.xPos = 0;
		focus = 4;
	}else if(returnGame.visible && Collide(returnGame, x, y))
	{
		bar.visible = true;
		bar.yPos = -37.5f;
		bar.xPos = -55;
		focus = 5;
	}else if(saveGame.visible && Collide(saveGame, x, y))
	{
		bar.visible = true;
		bar.yPos = -37.5f;
		bar.xPos = 55;
		focus = 6;
	}else{
		bar.visible = false;
		focus = 0;
	}
}

void MenuHandler::OnMouseUp(bool lmb, bool rmb)
{
	if(bSaveGame)
		return;
	if(!lmb)
		return;
	if(focus == 1)
	{
		//New Game: create a transition from this to LevelHandler
		TransitionHandler* t = new TransitionHandler;
		LevelHandler* game = new LevelHandler;
		game->Initialize();
		game->LoadLevel(1);
		t->Initialize();
		t->BeginTransition(this, game, 15);
		state = t;
	}else if(focus == 2)
	{
		//Load Game: change to load game dialog
		TransitionHandler* t = new TransitionHandler;
		LoadHandler* l = new LoadHandler;
		l->bInGame = bInGame;
		l->lvl = retLevel;
		l->Initialize();
		l->LoadLevel(1);
		t->Initialize();
		t->BeginTransition(this, l, 15);
		state = t;
	}else if(focus == 3)
	{
		//Story: transition to the story dialog
		TransitionHandler* t = new TransitionHandler;
		StoryHandler* s = new StoryHandler;
		s->bInGame = bInGame;
		s->lvl = retLevel;
		s->Initialize();
		s->LoadLevel(1);
		t->Initialize();
		t->BeginTransition(this, s, 15);
		state = t;
	}else if(focus == 4)
	{
		//Quit Game: set bRunning to false
		bRunning = false;
	}else if(focus == 5)
	{
		//Return to a game: only visible from in-game menu
		TransitionHandler* t = new TransitionHandler;
		t->Initialize();
		t->BeginTransition(this, retLevel, 15);
		state = t;
	}else if(focus == 6)
	{
		//Switch to Save Game mode
		bSaveGame = true;
		caretPos = 0;
	}
}

void MenuHandler::OnKeyDown(int key)
{
	if(bSaveGame)
	{
		int maxSize = 28;
		if(key > 0x2f && key < 0x3a && saveName.size() < maxSize)
		{
			char buf[2] = {0};
			buf[0] = (char)key;
			saveName.insert(caretPos, buf);
			caretPos++;
		}
		if(key > 0x40 && key < 0x5b && saveName.size() < maxSize)
		{
			char buf[2] = {0};
			buf[0] = (char)key - 'A' + 'a';
			saveName.insert(caretPos, buf);
			caretPos++;
		}
		if(key == VK_SPACE && saveName.size() < maxSize)
		{
			saveName.insert(caretPos, " ");
			caretPos++;
		}
		if(key == VK_BACK)
		{
			caretPos--;
			if(caretPos > -1)
			{
				saveName.erase(caretPos, 1);
			}
		}

		if(key == VK_LEFT)
			caretPos--;
		if(key == VK_RIGHT)
			caretPos++;

		if(caretPos < 0)
			caretPos = 0;
		if(caretPos > saveName.size())
			caretPos = saveName.size();

		if(key == VK_ESCAPE)
			bSaveGame = false;

		if(key == VK_RETURN)
		{
			//Save to specified file
			bSaveGame = false;

			string sFile = "Save/";
			sFile += saveName;
			sFile += ".sav";

			retLevel->SaveTo(sFile);
		}
	}
}
