// Demo2D.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "camera.h"
#include "texture.h"
#include "sprite.h"
#include "levelhandler.h"
#include "menuhandler.h"
#include "TextRenderer.h"

EventHandler* state;

#define FPS_SMOOTHING 1
float speedModifier = 0.85f;

int width = 640;
int height = 480;
int depth = 32;
bool bFullscreen = true;
int mouseX = 0;
int mouseY = 0;

list<float> lastM;

float m = 0;
float fps = 0;

unsigned int lastTime;

bool keys[256] = {0};
bool bRunning = true;

bool Collide(Sprite& s, float x, float y)
{
	float left;
    float right;
    float top;
    float bottom;

    left = s.xPos - s.width * 0.5f;
    right = s.xPos + s.width * 0.5f;
    top = s.yPos + s.height * 0.5f;
    bottom = s.yPos - s.height * 0.5f;

	if(x < left)
		return false;
	if(x > right)
		return false;
	if(y > top)
		return false;
	if(y < bottom)
		return false;

	return true;
}

bool Collide(Sprite& s1, Sprite& s2)
{
	float left1, left2;
    float right1, right2;
    float top1, top2;
    float bottom1, bottom2;

    left1 = s1.xPos - s1.width * 0.5f;
    left2 = s2.xPos - s2.width * 0.5f;
    right1 = s1.xPos + s1.width * 0.5f;
    right2 = s2.xPos + s2.width * 0.5f;
    top1 = s1.yPos - s1.height * 0.5f;
    top2 = s2.yPos - s2.height * 0.5f;
    bottom1 = s1.yPos + s1.height * 0.5f;
    bottom2 = s2.yPos + s2.height * 0.5f;

    if (bottom1 < top2)
		return false;
    if (top1 > bottom2) 
		return false;

    if (right1 < left2) 
		return false;
    if (left1 > right2)
		return false;

    return true;

}

void Render(HDC dc)
{
	//Calculate time difference
	unsigned int time = GetTickCount();
	lastM.push_back((float)(time - lastTime) / 1000.0f);
	if(lastM.size() > FPS_SMOOTHING)
		lastM.pop_front();

	//Average m over the last few frames to get smoother time-based movement
	list<float>::iterator it;
	float mTotal = 0;
	for(it = lastM.begin(); it != lastM.end(); it++)
	{
		mTotal += (*it);
	}
	m = (mTotal / (float)lastM.size());

	//Calculate FPS from that average
	fps = 1.0f / m;

	m *= speedModifier;

	lastTime = time;

	//Clear the back buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	//Update the current event handler and render
	state->OnMouse((float)mouseX / (float)width, 
		(float)(height - mouseY) / (float)height);	//Convert to OpenGL coordinates
	state->OnKeys(keys);
	state->Update();
	state->Render();

	//Present the back buffer
	SwapBuffers(dc);
}

LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
		case WM_SIZE:
			width = LOWORD(lParam);
			height = HIWORD(lParam);
			glViewport(0, 0, width, height);
			return 0;

        case WM_DESTROY:
            PostQuitMessage(0);
            return 0;

        case WM_PAINT:
            ValidateRect(hWnd, NULL);
            return 0;

		case WM_MOUSEMOVE:
			mouseX = LOWORD(lParam); 
			mouseY = HIWORD(lParam); 
			return 0;

		case WM_LBUTTONDOWN:
			state->OnMouseDown(true, false);
			return 0;

		case WM_RBUTTONDOWN:
			state->OnMouseDown(false, true);
			return 0;

		case WM_LBUTTONUP:
			state->OnMouseUp(true, false);
			return 0;

		case WM_RBUTTONUP:
			state->OnMouseUp(false, true);
			return 0;

		case WM_KEYDOWN:
			state->OnKeyDown(wParam);
			keys[wParam] = true;
			return 0;

		case WM_KEYUP:
			state->OnKeyUp(wParam);
			keys[wParam] = false;
			return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}

void ParseCommandLine(char* line)
{
	char buf[256] = {0};
	char l[256] = {0};
	strcpy(l, line);
	for(char* tok = strtok(&l[0], "-"); tok != NULL; tok = strtok(NULL, "-"))
	{
		string s = tok;
		if(s[0] == 'w')
		{
			sscanf(s.c_str(), "%s", buf);
			if(string(buf) == "win" || string(buf) == "windowed")
			{
				bFullscreen = false;
				continue;
			}
			int i = -1;
			sscanf(s.c_str(), "%s %d", buf, &i);
			if(string(buf) == "w" || string(buf) == "width")
			{
				if(i > 0)
					width = i;
			}
		}
		if(s[0] == 'h')
		{
			int i = -1;
			sscanf(s.c_str(), "%s %d", buf, &i);
			if(string(buf) == "h" || string(buf) == "height")
			{
				if(i > 0)
					height = i;
			}
		}
		if(s[0] == 'd')
		{
			int i = -1;
			sscanf(s.c_str(), "%s %d", buf, &i);
			if(string(buf) == "d" || string(buf) == "depth")
			{
				if(i > 0)
					depth = i;
			}
		}
	}
}

bool SwitchToFullscreen(unsigned int w, unsigned int h, unsigned char bpp)
{
	vector<DEVMODE> modes;
	vector<float> weights;

	//Enumerate the display modes
	int i = 0;
	DEVMODE d;
	while(EnumDisplaySettings(NULL, i, &d))
	{
		modes.push_back(d);	
		i++;
	}

	//Weigh each mode
	weights.reserve(modes.size());
	for(i = 0; i < modes.size(); i++)
	{
		float wDif = (float)modes[i].dmPelsWidth - (float)w;
		float hDif = (float)modes[i].dmPelsHeight - (float)h;
		float dDif = (float)modes[i].dmBitsPerPel - (float)bpp;
		if(wDif < 0)
			wDif *= -2;
		if(hDif < 0)
			hDif *= -1.5f;
		if(dDif < 0)
			dDif *= -2;
		weights.push_back((wDif + hDif) + (dDif * 50.0f));
	}

	//Find the node mode with the lowest weight
	int current = 0;
	for(i = 0; i < modes.size(); i++)
	{
		if(weights[i] < weights[current])
			current = i;
	}

	//Tell about the current mode
	/*char buf[256] = {0};
	string s = "Switching to: ";
	s += itoa(modes[current].dmPelsWidth, buf, 10);
	s += " x ";
	s += itoa(modes[current].dmPelsHeight, buf, 10);
	s += " x ";
	s += itoa(modes[current].dmBitsPerPel, buf, 10);
	MessageBox(NULL, s.c_str(), "Resolution Notifier", MB_OK);*/
	
	//Switch to fullscreen
	ZeroMemory(&d, sizeof(DEVMODE));
	d.dmSize = sizeof(DEVMODE);
	height = d.dmPelsHeight = modes[current].dmPelsHeight;
	width = d.dmPelsWidth = modes[current].dmPelsWidth;
	depth = d.dmBitsPerPel = modes[current].dmBitsPerPel;
	d.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
	return (ChangeDisplaySettings(&d, CDS_FULLSCREEN) == DISP_CHANGE_SUCCESSFUL);
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	//Parse command line
	ParseCommandLine(lpCmdLine);

	//Change resolution
	int style = WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	if(bFullscreen)
	{
		if(!SwitchToFullscreen(width, height, depth))
		{
			MessageBox(NULL, "Could not change the display settings\n"
				"Falling back to windowed mode.", "Error", MB_OK | MB_ICONEXCLAMATION);
			style = WS_OVERLAPPEDWINDOW;
		}
	}else{
		style = WS_OVERLAPPEDWINDOW;
	}
 	
	//Create a window
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      hInstance, NULL, NULL, NULL, NULL,
                      "MrGreen", NULL };
    RegisterClassEx(&wc);
	HWND hWnd = CreateWindowEx(NULL, "MrGreen", "Mr. Green: Attack of the Cabbages", style,
		0, 0, width, height, GetDesktopWindow(), 
		NULL, hInstance, NULL);

	//Update the window before OpenGL can start
	UpdateWindow(hWnd);

	//Setup pixel format
	HDC dc = GetDC(hWnd);
	PIXELFORMATDESCRIPTOR pfd = {sizeof(PIXELFORMATDESCRIPTOR),
		1, 
		PFD_DRAW_TO_WINDOW | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		depth,		//Colour buffer bits
		0,0,		//R bits/shift
		0,0,		//G bits/shift
		0,0,		//B bits/shift
		0,0,		//A bits/shift
		0,			//Accum buffer bits
		0,0,0,0,	//Accum RGBA bits
		24,			//Depth bits
		8,			//Stencil bits
		0,0,0,0,0,0};
	int format = ChoosePixelFormat(dc, &pfd);
	SetPixelFormat(dc, format, &pfd);

	TextRenderer::dc = dc;

	//Setup rendering context
	HGLRC rc = wglCreateContext(dc);
	wglMakeCurrent(dc, rc);

	//Clear for the first time
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	//Show the window
	ShowWindow(hWnd, SW_SHOW);

	//Initialize DevIL
	ilInit();

	//Enable some GL states
	glEnable(GL_DEPTH_TEST);

	//Load the first level
	state = new MenuHandler;
	state->Initialize();
	state->LoadLevel(1);

	lastTime = GetTickCount();

	//Main message loop
	MSG msg;
	while(bRunning)
	{
		Render(dc);
		while(PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
		{
			if(msg.message == WM_QUIT)
			{
				bRunning = false;
				break;
			}
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	//Cleanup
	MenuHandler::bkTex.Unload();
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(rc);
	ReleaseDC(hWnd, dc);
	DestroyWindow(hWnd);

	ChangeDisplaySettings(NULL, 0);

	return 0;
}



