// Ninja.cpp: implementation of the Ninja class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Sprite.h"
#include "Texture.h"
#include "Ninja.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Ninja::Ninja()
{

}

Ninja::~Ninja()
{

}

int Ninja::Move(float xPos, float yPos)
{
	//Invert U coordinate if necessary
	if(destX > sprite.xPos)
	{
		sprite.texture.uMax = -1;
	}else{
		sprite.texture.uMax = 1;
	}

	//Animate
	if(attackTimer > 0)
	{
		attackTimer -= 64.0f * m;

		float frame = (1.0f - attackTimer / 12.0f) * (float)(ninjaTex.size() - 1);
		sprite.texture = ninjaTex[(int)frame];
		if(attackTimer < 0)
		{
			if((xPos - sprite.xPos)*(xPos - sprite.xPos) + 
				(yPos - sprite.yPos)*(yPos - sprite.yPos) < 400.0f)
			return 10;
		}
		return 0;
	}else if(attackTimer > -25.0f)
	{
		attackTimer -= 64.0f * m;
		sprite.texture = ninjaTex[0];
		return 0;
	}

	//Move towards the point
	float xDist = destX - sprite.xPos;
	float yDist = destY - sprite.yPos;
	float mag = sqrt(xDist*xDist + yDist*yDist);
	//Change the point if it has been reached
	if(mag < 0.5f)
	{
		sprite.xPos = destX;
		sprite.yPos = destY;
		if(bDir)
		{
			destX = xPos + (float)(rand() % 10) - 5.0f;
			destY = sprite.yPos;
		}else{
			destX = sprite.xPos;
			destY = yPos  + (float)(rand() % 10) - 5.0f;
		}
		bDir = !bDir;
	}else if((xPos - sprite.xPos)*(xPos - sprite.xPos) + 
		(yPos - sprite.yPos)*(yPos - sprite.yPos) < 350.0f)
	{
		attackTimer = 12.0f;
		return 0;
	}else{
		//Reduce speed when too close to a ninja of a higher priority
		bool bFound = false;
		for(int i = 0; i < peers->size(); i++)
		{
			Ninja& p = *(*peers)[i];
			if(p.priority > priority)
			{
				if(Collide(p.sprite, sprite))
					bFound = true;
			}
		}
		if(bFound)
			speed = 1.2f;
		else
			speed = 6.4f;

		//Lerp between position and dest point
		float xLerp = sprite.xPos + (speed * m) * (destX - sprite.xPos);
		float yLerp = sprite.yPos + (speed * m) * (destY - sprite.yPos);
		xDist /= mag;
		yDist /= mag;
		sprite.xPos = xLerp;
		sprite.yPos = yLerp;
	}

	return 0;
}

void Ninja::Setup()
{
	destX = sprite.xPos;
	destY = sprite.yPos;
	speed = 5.4f;
	attackTimer = 0;
	static bool b;
	b = !b;
	bDir = b;
}

int Ninja::GetType()
{
	return 2;
}
